import 'dart:async';
import 'dart:convert';

import 'package:barcode_scanner/scanbot_sdk_models.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:logging/logging.dart';

import 'barcode_scanning_data.dart';
import 'license.dart';

class ScanbotBarcodeSdk {
  static const MethodChannel _channel = MethodChannel('barcode_scanner');
  static const MethodChannel sessionCallbackChannel =
      MethodChannel('batch_barcode_data_mapper');

  /// Initializes the Scanbot SDK with given [config]s.
  /// The Scanbot SDK must be initialized before using any other API methods.
  static Future initScanbotSdk(ScanbotSdkConfig config) async {
    try {
      LicenseRegistry.addLicense(licenses);
      var json = config.toJson();
      await _channel.invokeMethod('initScanbotSdk', json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  static bool wasCanceled(String result) {
    return result == 'CANCELED';
  }

  /// Returns the current license status of the Scanbot SDK.
  static Future<SdkLicenseStatus> getLicenseStatus() async {
    try {
      var licenseStatus = await _channel.invokeMethod('getLicenseStatus', {});
      var json = jsonDecode(licenseStatus);
      return SdkLicenseStatus.fromJson(json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Clear storage folder with barcode snapped images
  static Future cleanupBarcodeStorage() async {
    try {
      await _channel.invokeMethod('cleanupBarcodeStorage', {});
      return;
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Starts the Ready-to-Use Barcode Scanner UI as full-screen modal UI dialog.
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  static Future<BarcodeScanningResult> startBarcodeScanner(
      BarcodeScannerConfiguration config) async {
    try {
      var barcodeScanningResult =
          await _channel.invokeMethod('startBarcodeScanner', config.toJson());

      if (wasCanceled(barcodeScanningResult)) {
        return BarcodeScanningResult.canceled();
      }

      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      return BarcodeScanningResult.error(e.toString());
    }
  }

  static Handler? _handler;

  static void initHandlerForSession(
      MethodChannel sessionCallbackChannel, BarcodeDataFormatter? formatter) {
    if (formatter == null) {
      return;
    }
    _handler = (MethodCall call) async {
      if (call.method == 'getFormattedData') {
        var dataItem =
            await formatter(BarcodeItem.fromJson(jsonDecode(call.arguments)));
        return dataItem.toJson();
      }
    };
  }

  /// Starts the Ready-to-Use Batch Barcode Scanner UI as full-screen modal UI dialog.
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  static Future<BarcodeScanningResult> startBatchBarcodeScanner(
      BatchBarcodeScannerConfiguration config) async {
    try {
      var json = config.toJson();
      if (config.barcodeFormatter != null) {
        initHandlerForSession(sessionCallbackChannel, config.barcodeFormatter);
        sessionCallbackChannel.setMethodCallHandler(_handler);
        json['shouldUseBarcodeFormatter'] = true;
      } else {
        _handler = null;
      }

      var barcodeScanningResult =
          await _channel.invokeMethod('startBatchBarcodeScanner', json);
      _handler = null;

      if (wasCanceled(barcodeScanningResult)) {
        return BarcodeScanningResult.canceled();
      }

      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      return BarcodeScanningResult.error(e.toString());
    }
  }

  /// Detect barcodes  from image file uri
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  ///image [rotation]  between 0-360 degrees with 90 degree step
  /// [barcodeFormats] Format of barcode which  need to recognize
  /// [acceptedDocumentFormats] Accepted Format of Documents which  need to recognize
  /// [barcodeAdditionalParameters] Additional parameters for tweaking the detection of barcodes.
  static Future<BarcodeScanningResult> detectFromImageFile(
    Uri file, {
    List<BarcodeFormat>? barcodeFormats,
    List<AcceptedDocumentFormats>? acceptedDocumentFormats,
    BarcodeAdditionalParameters? barcodeAdditionalParameters,
  }) async {
    try {
      barcodeAdditionalParameters ??= BarcodeAdditionalParameters();
      acceptedDocumentFormats ??= <AcceptedDocumentFormats>[];
      barcodeFormats ??= const <BarcodeFormat>[];
      var detectFromImageParams = DetectFromImageParams(
        file,
        barcodeFormats,
        acceptedDocumentFormats,
        barcodeAdditionalParameters,
      );
      var barcodeScanningResult = await _channel.invokeMethod(
          'detectFromImageFile', detectFromImageParams.toJson());

      if (wasCanceled(barcodeScanningResult)) {
        return BarcodeScanningResult.canceled();
      }
      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      return BarcodeScanningResult.error(e.toString());
    }
  }
}

typedef Handler = Future Function(MethodCall call);
